###########################################################
## File Name: Chapter5_Grimm_Ram_Estabrook_OpenMx.R      ## 
## Read data                                             ##
## Fit Linear Growth Models with Covariates with OpenMx  ##
## By Kevin J. Grimm                                     ##
## R Version 3.1.0                                       ##
###########################################################


# Load the data
nlsy_math_wide<-read.table("C:/GRE2016/Data/nlsy_math_wide_R.dat", na.string='.')

# Give the variable names
names(nlsy_math_wide)<-c('id','female','lb_wght','anti_k1',
                         'math2','math3','math4','math5','math6','math7','math8',
                         'age2','age3','age4','age5','age6','age7','age8',
                         'men2','men3','men4','men5','men6','men7','men8',
                         'spring2','spring3','spring4','spring5','spring6','spring7','spring8',
                         'anti2','anti3','anti4','anti5','anti6','anti7','anti8')

# Attach Data
attach(nlsy_math_wide)

library(OpenMx)

lg.math.omx <- mxModel("Linear Growth with Time Invariant Covariates",
  type="RAM", mxData(observed=nlsy_math_wide, type="raw"),
  manifestVars=c("math2","math3","math4","math5","math6","math7","math8","lb_wght","anti_k1"),
  latentVars=c("eta_1","eta_2"),

# residual variance paths 
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c("psi_11", "psi_21", "psi_22")),

# factor loadings 
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2",  to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=TRUE, values=c(40, 4), labels=c("alpha_1","alpha_2")),

####Including Time-Invariant Covariates 
#Means 
mxPath(from="one", to=c("lb_wght","anti_k1"),
 arrows=1, free=TRUE, values=c(.1, 1), labels=c("omega_1","omega_2")),

# variances and covariance paths 
mxPath(from=c("lb_wght","anti_k1"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(.1, 0, 1), labels=c("phi_11", "phi_21", "phi_22")),

# regression coefficients
mxPath(from="lb_wght", to=c("eta_1", "eta_2"),
 arrows=1, free=TRUE, values=0, labels=c("beta_11", "beta_12")),

mxPath(from="anti_k1", to=c("eta_1", "eta_2"),
 arrows=1, free=TRUE, values=0, labels=c("beta_21", "beta_22"))

) # close model

lg.math.fit <- mxRun(lg.math.omx)

summary(lg.math.fit)


#free parameters:
#      name matrix     row     col     Estimate   Std.Error lbound ubound
#1  beta_11      A   eta_1 lb_wght -2.716294494 1.301890359              
#2  beta_12      A   eta_2 lb_wght  0.624659477 0.334625673              
#3  beta_21      A   eta_1 anti_k1 -0.550818926 0.232917913              
#4  beta_22      A   eta_2 anti_k1 -0.019288145 0.058977340              
#5       th      S   math2   math2 36.256219083 1.931951190              
#6   phi_11      S lb_wght lb_wght  0.073924044 0.003422635              
#7   phi_21      S lb_wght anti_k1  0.007405409 0.013537299              
#8   phi_22      S anti_k1 anti_k1  2.312192550 0.107062446              
#9   psi_11      S   eta_1   eta_1 63.064806696 5.966823609              
#10  psi_21      S   eta_1   eta_2 -0.078086267 1.281388626              
#11  psi_22      S   eta_2   eta_2  0.713153114 0.353764025              
#12 omega_1      M       1 lb_wght  0.080384346 0.008901304              
#13 omega_2      M       1 anti_k1  1.454447774 0.049780249              
#14 alpha_1      M       1   eta_1 36.289822774 0.497232673              
#15 alpha_2      M       1   eta_2  4.315158883 0.121967648              
#
#observed statistics:  4087 
#estimated parameters:  15 
#degrees of freedom:  4072 
#-2 log likelihood:  19570.17 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:  11426.171           19600.17                   NA
#BIC:  -8275.815           19672.75             19625.11
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 




lg.math.omx0 <- mxModel("Linear Growth with Time Invariant Covariates, effects to 0",
  type="RAM", mxData(observed=nlsy_math_wide, type="raw"), 
  manifestVars=c("math2","math3","math4","math5","math6","math7","math8","lb_wght","anti_k1"),
  latentVars=c("eta_1","eta_2"),

# residual variance paths 
mxPath(from=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=2, free=TRUE, values=60, labels="th"),

# latent variable variances and covariance 
mxPath(from=c("eta_1","eta_2"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(1,0,1), labels=c("psi_11", "psi_21", "psi_22")),

# regression weights
mxPath(from="eta_1", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=1),

mxPath(from="eta_2", to=c("math2","math3","math4","math5","math6","math7","math8"),
 arrows=1, free=FALSE, values=c(0, 1, 2, 3, 4, 5, 6)),

# means and intercepts
mxPath(from="one", to=c("eta_1","eta_2"),
 arrows=1, free=TRUE, values=c(40, 4), labels=c("alpha_1","alpha_2")),

####Including Time-Invariant Covariates 
#Means 
mxPath(from="one", to=c("lb_wght","anti_k1"),
 arrows=1, free=TRUE, values=c(.1, 1), labels=c("omega_1","omega_2")),

# variances and covariance paths 
mxPath(from=c("lb_wght","anti_k1"), connect='unique.pairs',
 arrows=2, free=TRUE, values=c(.1, 0, 1), labels=c("phi_11", "phi_21", "phi_22")),

# regression coefficients
mxPath(from="lb_wght", to=c("eta_1","eta_2"),
 arrows=1, free=FALSE, values=0),

mxPath(from="anti_k1", to=c("eta_1","eta_2"),
 arrows=1, free=FALSE, values=0)

) # close model

lg.math.fit0 <- mxRun(lg.math.omx0)

summary(lg.math.fit0)

#free parameters:
#      name matrix     row     col     Estimate   Std.Error lbound ubound
#1       th      S   math2   math2 36.230241928 1.831116589              
#2   phi_11      S lb_wght lb_wght  0.073924758 0.003422698              
#3   phi_21      S lb_wght anti_k1  0.007402833 0.013537428              
#4   phi_22      S anti_k1 anti_k1  2.312187886 0.107043820              
#5   psi_11      S   eta_1   eta_1 64.560095881 5.446678505              
#6   psi_21      S   eta_1   eta_2 -0.181272643 1.054660519              
#7   psi_22      S   eta_2   eta_2  0.732583571 0.309041342              
#8  omega_1      M       1 lb_wght  0.080384958 0.008901298              
#9  omega_2      M       1 anti_k1  1.454441781 0.049780067              
#10 alpha_1      M       1   eta_1 35.267383190 0.355425015              
#11 alpha_2      M       1   eta_2  4.339297618 0.088284735              
#
#observed statistics:  4087 
#estimated parameters:  11 
#degrees of freedom:  4076 
#-2 log likelihood:  19584.42 
#saturated -2 log likelihood:  NA 
#number of observations:  933 
#chi-square:  NA 
#p:  NA 
#Information Criteria: 
#     df Penalty Parameters Penalty Sample-Size Adjusted
#AIC:  11432.416           19606.42                   NA
#BIC:  -8288.924           19659.64              19624.7
#CFI: NA 
#TLI: NA 
#RMSEA:  NA 

