###########################################################
## File Name: Chapter5_Grimm_Ram_Estabrook_NLME.R        ## 
## Read data, Fit NLME Models with Covariates            ##
## Plot longitudinal data in R                           ##
## By Kevin J. Grimm                                     ##
## R Version 3.0.2                                       ##
###########################################################

install.packages("nlme")

library(nlme)

# Load the data
nlsy_math_long<-read.table("D:/GRE2016/Data/nlsy_math_long_R.dat", na.strings='.')

# Give the variable names
names(nlsy_math_long)<-c('id', 'female', 'lb_wght', 'anti_k1', 'math', 'grade', 'occ', 'age', 'men', 'spring', 'anti')

attach(nlsy_math_long)


#Using lme to fit Linear Growth Model to Mathematics Data with Time-Invariant Covariates
grade_c2 <- grade - 2
summary(lb_wght)
summary(anti_k1)

lg.math.lme <- lme(math ~ grade_c2 + anti_k1 + lb_wght + grade_c2*anti_k1 + grade_c2*lb_wght, random= ~ grade_c2 |id, data = nlsy_math_long, method="ML")
summary(lg.math.lme)

#Linear mixed-effects model fit by maximum likelihood
# Data: nlsy_math_long 
#       AIC     BIC   logLik
#  15943.14 16000.2 -7961.57
#
#Random effects:
# Formula: ~grade_c2 | id
# Structure: General positive-definite, Log-Cholesky parametrization
#            StdDev    Corr  
#(Intercept) 7.9413095 (Intr)
#grade_c2    0.8445042 -0.012
#Residual    6.0213584       
#
#Fixed effects: math ~ grade_c2 + anti_k1 + lb_wght + grade_c2 * anti_k1 + grade_c2 *      lb_wght 
#                    Value Std.Error   DF  t-value p-value
#(Intercept)      36.28987 0.4969765 1286 73.02131  0.0000
#grade_c2          4.31516 0.1207519 1286 35.73574  0.0000
#anti_k1          -0.55088 0.2327789  929 -2.36655  0.0182
#lb_wght          -2.71621 1.2953406  929 -2.09691  0.0363
#grade_c2:anti_k1 -0.01929 0.0589361 1286 -0.32731  0.7435
#grade_c2:lb_wght  0.62464 0.3335738 1286  1.87256  0.0614
# Correlation: 
#                 (Intr) grd_c2 ant_k1 lb_wgh g_2:_1
#grade_c2         -0.529                            
#anti_k1          -0.671  0.358                     
#lb_wght          -0.194  0.096 -0.025              
#grade_c2:anti_k1  0.343 -0.660 -0.529  0.026       
#grade_c2:lb_wght  0.091 -0.168  0.026 -0.532 -0.055
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.080147868 -0.525141672 -0.008659953  0.530785971  2.534567541 
#
#Number of Observations: 2221
#Number of Groups: 932 


#Using nlme to fit Linear Growth Model to Mathematics Data with Time-Invariant Covariates
lg.math.nlme <- nlme(math ~ (beta_01 + beta_11*lb_wght + beta_21*anti_k1 + d_1i) +
                    (beta_02 + beta_12*lb_wght + beta_22*anti_k1 + d_2i)*(grade-2),
        data=nlsy_math_long,
        fixed=beta_01+beta_11+beta_21+beta_02+beta_12+beta_22~1,
        random=d_1i+d_2i~1,
        group=~id,
        start=c(beta_01=30, beta_11=0, beta_21=0, beta_02=4, beta_12=0, beta_22=0),
        na.action="na.omit")

summary(lg.math.nlme)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ (beta_01 + beta_11 * lb_wght + beta_21 * anti_k1 + d_1i) + (beta_02 + beta_12 * lb_wght + beta_22 * anti_k1 + d_2i) * (grade - 2) 
# Data: nlsy_math_long 
#       AIC     BIC   logLik
#  15943.14 16000.2 -7961.57
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr  
#d_1i     7.9413082 d_1i  
#d_2i     0.8445044 -0.012
#Residual 6.0213584       
#
#Fixed effects: beta_01 + beta_11 + beta_21 + beta_02 + beta_12 + beta_22 ~ 1 
#           Value Std.Error   DF  t-value p-value
#beta_01 36.28987 0.4969765 1284 73.02130  0.0000
#beta_11 -2.71621 1.2953406 1284 -2.09691  0.0362
#beta_21 -0.55088 0.2327789 1284 -2.36655  0.0181
#beta_02  4.31516 0.1207520 1284 35.73573  0.0000
#beta_12  0.62464 0.3335738 1284  1.87256  0.0614
#beta_22 -0.01929 0.0589361 1284 -0.32731  0.7435
# Correlation: 
#        bet_01 bet_11 bet_21 bet_02 bet_12
#beta_11 -0.194                            
#beta_21 -0.671 -0.025                     
#beta_02 -0.529  0.096  0.358              
#beta_12  0.091 -0.532  0.026 -0.168       
#beta_22  0.343  0.026 -0.529 -0.660 -0.055
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.080147827 -0.525141637 -0.008659961  0.530785961  2.534567488 
#
#Number of Observations: 2221
#Number of Groups: 932 


#Using nlme to fit Linear Growth Model to Mathematics Data with Time-Invariant Covariate Effects Fixed at 0
lg.math.nlme0 <- nlme(math ~ (beta_01 + 0*lb_wght + 0*anti_k1 + d_1i) +
                    (beta_02 + 0*lb_wght + 0*anti_k1 + d_2i)*(grade-2),
        data=nlsy_math_long,
        fixed=beta_01+beta_02~1,
        random=d_1i+d_2i~1,
        group=~id,
        start=c(beta_01=30, beta_02=4),
        na.action="na.omit")

summary(lg.math.nlme0)

#Nonlinear mixed-effects model fit by maximum likelihood
#  Model: math ~ (beta_01 + 0 * lb_wght + 0 * anti_k1 + d_1i) + (beta_02 + 0 * lb_wght + 0 * anti_k1 + d_2i) * (grade - 2) 
# Data: nlsy_math_long 
#       AIC      BIC    logLik
#  15949.39 15983.62 -7968.693
#
#Random effects:
# Formula: list(d_1i ~ 1, d_2i ~ 1)
# Level: id
# Structure: General positive-definite, Log-Cholesky parametrization
#         StdDev    Corr  
#d_1i     8.0350367 d_1i  
#d_2i     0.8558992 -0.026
#Residual 6.0191141       
#
#Fixed effects: beta_01 + beta_02 ~ 1 
#           Value Std.Error   DF  t-value p-value
#beta_01 35.26736 0.3551567 1288 99.30083       0
#beta_02  4.33933 0.0873767 1288 49.66231       0
# Correlation: 
#        bet_01
#beta_02 -0.532
#
#Standardized Within-Group Residuals:
#         Min           Q1          Med           Q3          Max 
#-3.030277803 -0.525876473  0.001632866  0.540751597  2.542250168 
#
#Number of Observations: 2221
#Number of Groups: 932 


##LME4 
install.packages("lme4")

library(lme4)

lg.math.lmer <- lmer(math ~ 1 + grade_c2 + lb_wght + anti_k1 + grade_c2*lb_wght + grade_c2*anti_k1 
                              + (1 + grade_c2 | id), 
                              data = nlsy_math_long, 
                              REML = FALSE)
summary(lg.math.lmer)

#Linear mixed model fit by maximum likelihood  ['lmerMod']
#Formula: math ~ 1 + grade_c2 + lb_wght + anti_k1 + grade_c2 * lb_wght +  
#    grade_c2 * anti_k1 + (1 + grade_c2 | id)
#   Data: nlsy_math_long
#
#     AIC      BIC   logLik deviance df.resid 
# 15943.1  16000.2  -7961.6  15923.1     2211 
#
#Scaled residuals: 
#     Min       1Q   Median       3Q      Max 
#-3.08015 -0.52514 -0.00866  0.53079  2.53457 
#
#Random effects:
# Groups   Name        Variance Std.Dev. Corr 
# id       (Intercept) 63.0644  7.9413        
#          grade_c2     0.7132  0.8445   -0.01
# Residual             36.2568  6.0214        
#Number of obs: 2221, groups: id, 932
#
#Fixed effects:
#                 Estimate Std. Error t value
#(Intercept)      36.28987    0.49630   73.12
#grade_c2          4.31516    0.12059   35.78
#lb_wght          -2.71621    1.29359   -2.10
#anti_k1          -0.55088    0.23246   -2.37
#grade_c2:lb_wght  0.62464    0.33312    1.88
#grade_c2:anti_k1 -0.01929    0.05886   -0.33
#
#Correlation of Fixed Effects:
#            (Intr) grd_c2 lb_wgh ant_k1 gr_2:_
#grade_c2    -0.529                            
#lb_wght     -0.194  0.096                     
#anti_k1     -0.671  0.358 -0.025              
#grd_c2:lb_w  0.091 -0.168 -0.532  0.026       
#grd_c2:nt_1  0.343 -0.660  0.026 -0.529 -0.055

