ODS HTML CLOSE;
ODS LISTING;


DATA nlsy_math_long;
	INFILE 'C:\GRE2016\Data\nlsy_math_long_R.dat';
	INPUT 
		id	female 		lb_wght		anti_k1

		math 	grade		occ						age
		men		spring		anti;
RUN;

PROC FREQ;
	TABLES anti_k1;
RUN;

DATA nlsy_math_long1;
	SET nlsy_math_long;
	grade_c2 = grade - 2;
RUN;

*Linear Growth Curve without time-invariant covariates;
PROC MIXED DATA = nlsy_math_long1 NOCLPRINT COVTEST METHOD = ML;
	TITLE1 'Linear Growth Model';
	CLASS id;
	MODEL math = grade_c2 /SOLUTION DDFM=bw NOTEST CHISQ;
	RANDOM intercept grade_c2/SUBJECT=id type=UN GCORR;
RUN;
/*
                                 Estimated G Correlation Matrix

                       Row    Effect         id           Col1        Col2

                         1    Intercept        201      1.0000    -0.02624
                         2    grade_c2         201    -0.02624      1.0000

                                 Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm     Subject    Estimate       Error     Value        Pr Z

               UN(1,1)        id        64.5585      5.6591     11.41      <.0001
               UN(2,1)        id        -0.1804      1.1500     -0.16      0.8753
               UN(2,2)        id         0.7323      0.3273      2.24      0.0126
               Residual                 36.2308      1.8666     19.41      <.0001

                                        Fit Statistics

                             -2 Log Likelihood             15937.4
                             AIC (smaller is better)       15949.4
                             AICC (smaller is better)      15949.4
                             BIC (smaller is better)       15978.4

                                   Solution for Fixed Effects

                                         Standard
                Effect       Estimate       Error      DF    t Value    Pr > |t|

                Intercept     35.2674      0.3550     931      99.35      <.0001
                grade_c2       4.3393     0.08734    1288      49.69      <.0001
*/

*Linear Growth Curve with lb_wght & anti_k1 as covariates;
PROC MIXED DATA = nlsy_math_long1 NOCLPRINT COVTEST METHOD = ML;
	TITLE1 'Linear Growth Model with Covariates';
	CLASS id;
	MODEL math = grade_c2 lb_wght lb_wght*grade_c2 anti_k1 anti_k1*grade_c2/SOLUTION DDFM=bw NOTEST CHISQ;
	RANDOM intercept grade_c2/SUBJECT=id type=UN GCORR;
RUN;

/*
                                Estimated G Correlation Matrix

                       Row    Effect         id           Col1        Col2

                         1    Intercept        201      1.0000    -0.01162
                         2    grade_c2         201    -0.01162      1.0000

                                 Covariance Parameter Estimates

                                                   Standard         Z
               Cov Parm     Subject    Estimate       Error     Value        Pr Z

               UN(1,1)        id        63.0631      5.6094     11.24      <.0001
               UN(2,1)        id       -0.07790      1.1446     -0.07      0.9457
               UN(2,2)        id         0.7131      0.3264      2.18      0.0145
               Residual                 36.2572      1.8683     19.41      <.0001

                                        Fit Statistics

                             -2 Log Likelihood             15923.1
                             AIC (smaller is better)       15943.1
                             AICC (smaller is better)      15943.2
                             BIC (smaller is better)       15991.5

                                  Solution for Fixed Effects

                                            Standard
            Effect              Estimate       Error      DF    t Value    Pr > |t|

            Intercept            36.2899      0.4963     929      73.12      <.0001
            grade_c2              4.3152      0.1206    1286      35.78      <.0001
            lb_wght              -2.7162      1.2936     929      -2.10      0.0360
            grade_c2*lb_wght      0.6246      0.3331    1286       1.88      0.0610
            anti_k1              -0.5509      0.2325     929      -2.37      0.0180
            grade_c2*anti_k1    -0.01929     0.05886    1286      -0.33      0.7432
*/

*Fitting a Linear Growth Model with time-invariant covariates in SAS using NLMIXED;
PROC NLMIXED DATA = nlsy_math_long;
	b_1i = beta_01 + beta_11*lb_wght + beta_21*anti_k1 + d_1i;
	b_2i = beta_02 + beta_12*lb_wght + beta_22*anti_k1 + d_2i;

	traject = b_1i + b_2i * (grade - 2);
	MODEL math ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
									  c_21, v_2])
	SUBJECT = id;
	PARMS	 
       beta_01 = 35   beta_02 = 4
       beta_11 = 0    beta_12 = 0    beta_21 = 0    beta_22 = 0
       v_1 = 70       v_2 = 1        c_21 = 0       v_u = 36; 
RUN;

/*
                                        Fit Statistics

                           -2 Log Likelihood                  15923
                           AIC (smaller is better)            15943
                           AICC (smaller is better)           15943
                           BIC (smaller is better)            15992


                                     Parameter Estimates

                      Standard
 Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

 beta_01     36.2899    0.4968   930    73.05    <.0001    0.05   35.3150   37.2648  0.000086
 beta_02      4.3152    0.1218   930    35.42    <.0001    0.05    4.0761    4.5543  0.000739
 beta_11     -2.7160    1.2939   930    -2.10    0.0361    0.05   -5.2554   -0.1766  0.000129
 beta_12      0.6246    0.3334   930     1.87    0.0613    0.05  -0.02972    1.2789  0.000045
 beta_21     -0.5509    0.2325   930    -2.37    0.0180    0.05   -1.0072  -0.09463  -0.00054
 beta_22    -0.01929   0.05892   930    -0.33    0.7435    0.05   -0.1349   0.09635  -0.00028
 v_1         63.0664    5.6097   930    11.24    <.0001    0.05   52.0572   74.0756  7.866E-6
 v_2          0.7133    0.3264   930     2.19    0.0291    0.05   0.07270    1.3539   0.00051
 c_21       -0.07895    1.1447   930    -0.07    0.9450    0.05   -2.3254    2.1676  -0.00054
 v_u         36.2569    1.8683   930    19.41    <.0001    0.05   32.5903   39.9235  0.000269
*/

*Fitting a Linear Growth Model with time-invariant covariates in SAS using NLMIXED - paths fixed at 0;
PROC NLMIXED DATA = nlsy_math_long;
	b_1i = beta_01 + 0*lb_wght + 0*anti_k1 + d_1i;
	b_2i = beta_02 + 0*lb_wght + 0*anti_k1 + d_2i;

	traject = b_1i + b_2i * (grade - 2);
	MODEL math ~ NORMAL(traject, v_u);
	RANDOM d_1i d_2i ~ NORMAL([0,0], [v_1,
									  c_21, v_2])
	SUBJECT = id;
	PARMS	 
       beta_01 = 35   beta_02 = 4
       v_1 = 70       v_2 = 1        c_21 = 0       v_u = 36; 
RUN;

/*
                                        Fit Statistics

                           -2 Log Likelihood                  15937
                           AIC (smaller is better)            15949
                           AICC (smaller is better)           15949
                           BIC (smaller is better)            15978


                                     Parameter Estimates

                      Standard
 Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

 beta_01     35.2674    0.3554   930    99.23    <.0001    0.05   34.5699   35.9649  -0.00005
 beta_02      4.3393   0.08831   930    49.14    <.0001    0.05    4.1660    4.5126  -0.00016
 v_1         64.5618    5.6594   930    11.41    <.0001    0.05   53.4552   75.6685  -3.56E-6
 v_2          0.7326    0.3273   930     2.24    0.0254    0.05   0.09026    1.3749   0.00016
 c_21        -0.1815    1.1501   930    -0.16    0.8746    0.05   -2.4385    2.0755  -2.94E-6
 v_u         36.2297    1.8665   930    19.41    <.0001    0.05   32.5666   39.8928  -1.13E-6
*/


*Predicted Trajectories using Simulation;
PROC MEANS DATA = nlsy_math_long;
	VAR lb_wght anti_k1;
RUN;
*Means are 0.08 & 1.42;

DATA sim_results;
	
	beta_01 =    36.2899;
	beta_02 =     4.3152;
	beta_11 =    -2.7160;
	beta_12 =     0.6246;
	beta_21 =    -0.5509;
	beta_22 =   -0.01929;

	DO id = 1 to 2;
		IF id = 1 THEN lb_wght = 0;
		IF id = 1 THEN anti_k1 = 1.42;

		IF id = 2 THEN lb_wght = 1;
		IF id = 2 THEN anti_k1 = 1.42;

		b_1i = beta_01 + beta_11 * lb_wght + beta_21 * anti_k1;
		b_2i = beta_02 + beta_12 * lb_wght + beta_22 * anti_k1;

		DO grade = 2 to 8;
			math = b_1i + b_2i * (grade - 2);
			OUTPUT;
		END;
	END;
RUN;

*Plotting Scores over time;
GOPTIONS
	FTEXT = ARIAL
	CTEXT = BLACK
	HTEXT = 1.5;
PROC GPLOT DATA = sim_results;
	SYMBOL1
		Interpol=join
		Color=black
		Repeat=1
		Value=none
		Height=.5
		Width=4
        Line=1;
	SYMBOL2
		Interpol=join
		Color=black
		Repeat=1
		Value=none
		Height=.5
		Width=4
        Line=2;

	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'PIAT Mathematics')
		ORDER = (0 to 90 BY 10)
		MINOR = none;
	AXIS2
		LABEL = (F=ARIAL H = 2 'Grade')
		ORDER = (2 to 8 BY 1)
		OFFSET= (5 PCT, 5 PCT)
		MINOR = none;

	PLOT math * grade = id/NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2;
RUN;


DATA sim_results;
	
	beta_01 =    36.2899;
	beta_02 =     4.3152;
	beta_11 =    -2.7160;
	beta_12 =     0.6246;
	beta_21 =    -0.5509;
	beta_22 =   -0.01929;

	DO id = 1 to 3;
		IF id = 1 THEN lb_wght = 0;
		IF id = 1 THEN anti_k1 = 0;

		IF id = 2 THEN lb_wght = 0;
		IF id = 2 THEN anti_k1 = 1.42;

		IF id = 3 THEN lb_wght = 0;
		IF id = 3 THEN anti_k1 = 2.92;

		b_1i = beta_01 + beta_11 * lb_wght + beta_21 * anti_k1;
		b_2i = beta_02 + beta_12 * lb_wght + beta_22 * anti_k1;

		DO grade = 2 to 8;
			math = b_1i + b_2i * (grade - 2);
			OUTPUT;
		END;
	END;
RUN;

*Plotting Scores over time;
GOPTIONS
	FTEXT = ARIAL
	CTEXT = BLACK
	HTEXT = 1.5;
PROC GPLOT DATA = sim_results;
	SYMBOL1
		Interpol=join
		Color=black
		Repeat=1
		Value=none
		Height=.5
		Width=4
        Line=2;
	SYMBOL2
		Interpol=join
		Color=black
		Repeat=1
		Value=none
		Height=.5
		Width=4
        Line=1;
	SYMBOL3
		Interpol=join
		Color=black
		Repeat=1
		Value=none
		Height=.5
		Width=4
        Line=2;

	AXIS1
		LABEL = (F=ARIAL H = 2 A=90 'PIAT Mathematics')
		ORDER = (0 to 90 BY 10)
		MINOR = none;
	AXIS2
		LABEL = (F=ARIAL H = 2 'Grade')
		ORDER = (2 to 8 BY 1)
		OFFSET= (5 PCT, 5 PCT)
		MINOR = none;

	PLOT math * grade = id/NOLEGEND VAXIS= AXIS1 HAXIS=AXIS2;
RUN;
